@@ -10,7 +10,12 @@
 #        returns its error conditions, or upgrade to Perl 5.6.0 or
 #        greater, which has its own bugs.
 
-package HTML::Mason;
+#
+# A much simpler alternative to this is to simply use the
+# HTML::Mason::Request::WithApacheSession code available from CPAN.
+#
+
+package MyMason::MyApp;
 
 # Bring in main Mason package.
 use HTML::Mason;
@@ -19,7 +24,6 @@ use HTML::Mason;
 # Uncomment the second line (and comment the first) to use
 # Apache::Request instead of CGI.pm to parse arguments.
 use HTML::Mason::ApacheHandler;
-# use HTML::Mason::ApacheHandler (args_method=>'mod_perl');
 
 use strict;
 
@@ -36,18 +40,11 @@ use strict;
    use Apache::Session::File 1.50;
 }
 
-# Create Mason objects
-#
-my $parser = new HTML::Mason::Parser;
-my $interp = new HTML::Mason::Interp (parser=>$parser,
-                                      comp_root=>'<component root>',
-                                      data_dir=>'<data directory>');
-my $ah = new HTML::Mason::ApacheHandler (interp=>$interp);
 
-# Activate the following if running httpd as root (the normal case).
-# Resets ownership of all files created by Mason at startup.
+# Create ApacheHandler object
 #
-#chown (Apache->server->uid, Apache->server->gid, $interp->files_written);
+my $ah = new HTML::Mason::ApacheHandler( comp_root => '<component root>',
+                                         data_dir => '<data directory>' );
 
 sub handler
 {
@@ -98,3 +95,14 @@ sub handler
 }
 
 1;
+
+
+__END__
+
+In your httpd.conf, add something like this:
+
+ PerlRequire /path/to/handler.pl
+ <FilesMatch "\.html$">
+   SetHandler perl-script
+   PerlHandler MyMason::MyApp
+ </FilesMatch>
