@@ -28,30 +28,25 @@ use HTML::Mason::MethodMaker
 		    ]
       );
 
-=pod
-
-=begin for reference
-
-__PACKAGE__->valid_params
-    (
-     attr               => {type => HASHREF, default => {}, public => 0},
-     code               => {type => CODEREF, public => 0, public => 0},
-     load_time          => {type => SCALAR,  optional => 1, public => 0},
-     declared_args      => {type => HASHREF, default => {}, public => 0},
-     dynamic_subs_init  => {type => CODEREF, default => sub {}, public => 0},
-     flags              => {type => HASHREF, default => {}, public => 0},
-     comp_id            => {type => SCALAR,  optional => 1, public => 0},
-     methods            => {type => HASHREF, default => {}, public => 0},
-     mfu_count          => {type => SCALAR,  default => 0, public => 0},
-     object_size        => {type => SCALAR,  default => 0, public => 0},
-     parser_version     => {type => SCALAR,  optional => 1, public => 0}, # allows older components to be instantied
-     compiler_id        => {type => SCALAR,  optional => 1, public => 0},
-     subcomps           => {type => HASHREF, default => {}, public => 0},
-    );
-
-=end
-
-=cut
+# for reference later
+# 
+# __PACKAGE__->valid_params
+#     (
+#      attr               => {type => HASHREF, default => {}, public => 0},
+#      code               => {type => CODEREF, public => 0, public => 0},
+#      load_time          => {type => SCALAR,  optional => 1, public => 0},
+#      declared_args      => {type => HASHREF, default => {}, public => 0},
+#      dynamic_subs_init  => {type => CODEREF, default => sub {}, public => 0},
+#      flags              => {type => HASHREF, default => {}, public => 0},
+#      comp_id            => {type => SCALAR,  optional => 1, public => 0},
+#      methods            => {type => HASHREF, default => {}, public => 0},
+#      mfu_count          => {type => SCALAR,  default => 0, public => 0},
+#      object_size        => {type => SCALAR,  default => 0, public => 0},
+#      parser_version     => {type => SCALAR,  optional => 1, public => 0}, # allows older components to be instantied
+#      compiler_id        => {type => SCALAR,  optional => 1, public => 0},
+#      subcomps           => {type => HASHREF, default => {}, public => 0},
+#     );
+# 
 
 my %defaults = ( attr              => {},
                  declared_args     => {},
@@ -119,8 +114,34 @@ sub run {
 
     $self->{mfu_count}++;
 
+    return $self->{code}->(@_) unless $self->{filter};
+
+    my $req = HTML::Mason::Request->instance;
+
+    $req->push_filter_buffer( filter => $self->{filter},
+                              filter_args => \@_ );
+
+    my @r;
+
+    my $wantarray = wantarray;
+    my @result;
     # Note: this must always preserve calling wantarray() context
-    return $self->{code}->(@_);
+    eval {
+        if ($wantarray) {
+            @result = $self->{code}->(@_);;
+        } elsif (defined $wantarray) {
+            $result[0] = $self->{code}->(@_);;
+        } else {
+            $self->{code}->(@_);
+        }
+    };
+
+    $req->flush_buffer;
+    $req->pop_buffer_stack;
+
+    die $@ if $@;
+
+    return $wantarray ? @result : defined $wantarray ? $result[0] : undef;
 }
 
 sub dynamic_subs_init {
@@ -153,6 +174,8 @@ sub persistent { 0 }
 #
 sub is_subcomp { 0 }
 
+sub is_method { 0 }
+
 #
 # Only true in FileBased subclass.
 #
@@ -312,6 +335,12 @@ sub object_file {
     return $self->interp->object_file($self);
 }
 
+# For backwards compatibility with 1.0x
+sub create_time {
+    my $self = shift;
+    return $self->load_time(@_);
+}
+
 1;
 
 __END__
@@ -398,10 +427,15 @@ attribute does not exist.
 Returns true if the specified attribute exists in this component or
 one of its parents, undef otherwise.
 
-=item load_time
+=item call_method (name, args...)
 
-Returns the time (in Perl time() format) when this component object
-was created.
+Looks for the specified user-defined method in this component and its
+parents, calling the first one found. Dies with an error if not found.
+Methods are declared in the C<E<lt>%methodE<gt>> section.
+
+=item create_time
+
+A synonym for L<load_time|HTML::Mason::Component/item_load_time> (deprecated).
 
 =item declared_args
 
@@ -445,24 +479,39 @@ Unlike attributes, flags values do not get inherited from parent components.
 
 =item is_subcomp
 
-Returns true if this is a subcomponent of another component.
+Returns true if this is a subcomponent of another component.  For
+historical reasons, this returns true for both methods and
+subcomponents.
+
+=item is_method
+
+Returns true if this is a method.
 
 =item is_file_based
 
 Returns true if this component was loaded from a source or object
 file.
 
-=item call_method (name, args...)
+=for html <a name="item_load_time"></a>
 
-Looks for the specified user-defined method in this component and its
-parents, calling the first one found. Dies with an error if not found.
-Methods are declared in the C<E<lt>%methodE<gt>> section.
+=item load_time
+
+Returns the time (in Perl time() format) when this component object
+was created.
 
 =item method_exists (name)
 
 Returns true if the specified user-defined method exists in this
 component or one of its parents, undef otherwise.
 
+=item methods
+
+This method works exactly like the
+L<subcomps|HTML::Mason::Component/item_subcomps> method, but it
+returns methods, not subcomponents.
+
+Methods are declared in C<E<lt>%methodE<gt>> sections.
+
 =item name
 
 Returns a short name of the component.  For file-based components this
@@ -495,6 +544,8 @@ args...)>, but returns the method output as a string instead of
 printing it. (Think sprintf versus printf.) The method's return value,
 if any, is discarded.
 
+=for html <a name="item_subcomps"></a>
+
 =item subcomps
 
 With no arguments, returns a hashref containing the subcomponents defined
@@ -508,13 +559,6 @@ or undef if no such subcomponent exists. e.g.
 
 Subcomponents are declared in C<E<lt>%defE<gt>> sections.
 
-=item methods
-
-This method works exactly like the subcomps method, but it returns
-methods, not subcomponents.
-
-Methods are declared in C<E<lt>%methodE<gt>> sections.
-
 =item title
 
 Returns a printable string denoting this component.  It is intended to
